#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Wed Mar 27 13:00:01 2024
@auteur: david ALBERTO
(www.astrolabe-science.fr)
Ce script trace les abondances naturelles isotopiques des éléments,
sous forme d'histogrammes, réunis dans un tableau périodique.
Les données isotopiques sont fournies par le module mendeleev
(https://mendeleev.readthedocs.io/)
"""
import matplotlib.pyplot as plt # graphiques
import numpy as np # gestion de séries de données
import pandas as pd # lecture de fichier csv et gestion de tableaux de données
from mendeleev import element # récupération du symbole de l'élément
from mendeleev.fetch import fetch_table # import des données isotopiques
from PIL import Image # insertion du logo CC

 # Paramètres personnalisables-----
liste_Z = range(1,55)
couleur_histo = 'skyblue'
couleur_magique = 'lightcoral'
couleur_doublemagique = 'blueviolet'
largeur = 0.2  # largeur des rectangles
plt.rcParams["font.family"] = "Roboto Condensed"
plt.rcParams["font.size"] = 8
NomFichier = "isotopesTableauPeriodique"
# ---------------------------------
liste_magique = [2, 8, 20, 28, 50, 82, 126]
#  import des données sur les éléments :
isotopes = fetch_table('isotopes', index_col='id') # récupération des données
# ---------------------------------
ptable  =  fetch_table('elements')
isotopes  =  fetch_table('isotopes', index_col = 'id')
colonnes = ['atomic_number','mass_number','abundance','is_radioactive']
donnees = isotopes[colonnes]
donnees = donnees[donnees.is_radioactive == False] # supprime les instables

# ---------------------------------
#  import du fichier contenant les coordonnées des éléments dans le tableau :
coord = pd.read_csv("donnees_elements_chimiques.csv", sep='\t',
                    usecols=['Z', 'lig', 'col'])
# ---------------------------------
#   FONCTIONS

def data(Z):
    """
    Parameters
    ----------
    Z : TYPE : (entier) : numéro atomique
    Returns
    -------
    liste x : nombres de masse des isotopes de l'élément.
    liste y = abondance des isotopes.
    """
    data = donnees[donnees.atomic_number == Z]
    data = data.dropna()  # supprime les données absentes
    liste_A = data.mass_number
    liste_abo = data.abundance
    return liste_A, liste_abo

def est_magique(Z,N):
    """
    Parameters
    ----------
    Z : TYPE entier
        DESCRIPTION = numéro atomique
    A : TYPE : entier
        DESCRIPTION nombre de neutrons
    Returns
    -------
    booléen : True si le noyau est magique.
    """
    if Z in liste_magique:
        magique = True
    elif N in liste_magique:
        magique = True
    else:magique = False
    return magique

def double_magique(Z,N):
    """
    Parameters
    ----------
    Z : TYPE entier
        DESCRIPTION = numéro atomique
    A : TYPE : entier
        DESCRIPTION nombre de neutrons
    Returns
    -------
    booléen : True si le noyau est doublement magique.
    """
    if Z in liste_magique and N in liste_magique:
        magique = True
    else:
        magique = False
    return magique

def liste_couleurs(Z, liste_A):
    """
    Parameters
    ----------
    Z : TYPE entier DESCRIPTION : numéro atomique de l'élément.
    liste_A : TYPE : liste de strings
        DESCRIPTION. : liste de nombres de masse des isotopes
    Returns
    -------
    liste_c : TYPE : liste de strings
        DESCRIPTION. : couleurs à affecter aux isotopes de l'élément.
    """
    liste_c = []
    for i in range(len(liste_A)):
        N = liste_A[i] - Z # nombre de neutrons
        if double_magique(Z, N):
            coul = couleur_doublemagique
        elif est_magique(Z, N):
            coul = couleur_magique
        else:
            coul = couleur_histo
        liste_c.append(coul)
    return liste_c

def histogramme(Z, ax):
    """
    Parameters
    ----------
    Z : TYPE : entier
        DESCRIPTION : numéro atomique
    Returns
    -------
    Trace l'histogramme des isotopes stables et leur abondance.
    """
    # paramètres du graphique
    ax.tick_params(axis='both', which='major', pad=1)# réduire l'espace des xticklabels
    ax.set_yticks([])
    x, y = data(Z)
    x = list(x)
    y = list(y)
    #  nom de l'élément :
    # nom = noms[Z-1]
    # titre avec le n° atomique et le nom pour chaque graphe :
    # ax.set_title(f"{Z}-{nom}", c = 'k', y = 0.98, alpha = 1)
    # n° atomique dans un coin du graphe :
    ax.text(0.01, 0.98, Z, c='gray', alpha=0.5, zorder=0, ha='left',
            va = 'top', transform = ax.transAxes, fontsize=20)
    # récupérer le symbole de l'élément :
    symbole = element(Z).symbol
    ax.text(0.5, 0.5, symbole, c='gray', alpha=0.5, zorder=0,
            transform = ax.transAxes, fontsize=40, ha='center')
    if len(y) > 0:# éviter une erreur si données absentes
        if len(x)==1:
            ax.set_xlim(x[0]-3, x[0]+3)# évite un rectangle trop large
        else:
            ax.set_xlim(x[0]-1, x[-1]+1)
        ax.set_ylim(0, max(y)+10) # laisse la place pour le %
        ecart = max(x) - min(x)
        if ecart <= 5:# pour graduer de 2 en 2 si trop d'écart
            ax.set_xticks(range(min(x), max(x)+1)) # graduations entières en x
        else:
            ax.set_xticks(range(min(x), max(x)+1,2)) # graduations 2 en 2
        couleur = liste_couleurs(Z, x) # liste de couleurs pour les rectangles
        ##### TRACÉ DE L'HISTOGRAMME -----------------------------
        ax.bar(x, y, width = largeur, ec='k', color = couleur,
               lw=0.5)
        # affichage des % :
        labels = [f"{y[i]}%" for i in range(len(y))] # les %
        """
        Création d'une liste de couleurs pour les pourcentages (colorés si 
        l'isotope est magique, noir sinon), en remplaçant la couleur bleue
        dans la liste 'couleur' par noir.
        """
        couleurs_labels = list(map(lambda x: x.replace(couleur_histo, 'k'), couleur))
        for i in range(len(x)):
            ax.text(x[i], y[i], labels[i], ha = 'center', va = 'bottom',
                    fontsize=5, rotation=90, color = couleurs_labels[i]
                    )

def coordonnees(Z):
    """
    renvoie les coordonnées (tuple) d'un élément de symbole donné.
    """
    x = coord.loc[coord.Z == Z, 'col'].values[0]
    y = coord.loc[coord.Z == Z, 'lig'].values[0]
    return x, y

# ---------------------------------------------------

noms = ['Hydrogène','Hélium','Lithium','Béryllium','Bore','Carbone','Azote','Oxygène',
      'Fluor','Néon','Sodium','Magnésium','Aluminium','Silicium','Phosphore','Soufre',
      'Chlore','Argon','Potassium','Calcium','Scandium','Titane','Vanadium','Chrome',
      'Manganèse','Fer','Cobalt','Nickel','Cuivre','Zinc','Gallium','Germanium',
      'Arsenic','Sélénium','Brome','Krypton','Rubidium','Strontium','Yttrium',
      'Zirconium','Niobium','Molybdène','Technétium','Ruthénium','Rhodium',
      'Palladium','Argent','Cadmium','Indium',
'Étain','Antimoine','Tellure','Iode','Xénon','Césium','Baryum','Lanthane','Cérium',
'Praséodyme','Néodyme','Prométhium','Samarium','Europium','Gadolinium','Terbium',
'Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium','Tantale',
'Tungstène','Rhénium','Osmium','Iridium','Platine','Or','Mercure','Thallium','Plomb',
'Bismuth','Polonium','Astate','Radon','Francium','Radium','Actinium','Thorium','Protactinium',
'Uranium','Neptunium','Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium',
'Fermium','Mendelevium','Nobelium','Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium',
'Hassium','Meitnerium','Darmstadtium','Roentgenium','Copernicium','Nihonium','Flerovium',
'Moscovium','Livermorium','Tennesse','Oganesson',
] # (le module mendeleev n'a que les noms des éléments en anglais)
# ---------------------------------------------------

fig, axes = plt.subplots(nrows=5, ncols=18, figsize = (59.4 / 2.54, 42 / 2.54), # format de la figure
                         facecolor = 'lavender', # couleur de fond de la figure
                          tight_layout = True # marge réduites
                         )

# effacement des fenetres graphiques inutiles :
for x in range(1,17):
    for y in range(1):
        axes[y, x].axis('off')
for x in range(2,12):
    for y in range(1, 3):
        axes[y, x].axis('off')

# boucle sur tous les éléments de la liste :
for Z in liste_Z:
    col, ligne = coordonnees(Z)
    axe = axes[ligne-1, col-1]
    histogramme(Z, axe)

fig.text(0.45,0.9,"Abondance naturelle des isotopes stables",
         fontsize=50, ha='center', fontweight='bold',
         transform = fig.transFigure)

# légende :
fig.text(0.2,0.45, "noyaux magiques", fontsize=10,
         ha='left', bbox = dict(facecolor=couleur_magique))
fig.text(0.2,0.425, "noyaux doublement magiques", fontsize=10,
         ha='left', bbox = dict(facecolor=couleur_doublemagique, alpha=0.6))


# logo Creative Commons et signature :
axe = axes[2, 11]
ax2 = axe.inset_axes([0, -0.025, 1, 0.2], transform=axe.transAxes)# création d'un axe dans un autre, pour accueillir le logo
ax2.set(xticks=[], yticks=[])
img = np.asarray(Image.open('by-nc-sa.png'))
imgplot = ax2.imshow(img)
axe.text(0.7,0.18, "David Alberto\n(www.astrolabe-science.fr)", transform = axe.transAxes,
          ha='left', va='bottom', fontsize=10, rotation = 90)

fig.savefig(NomFichier+'.png',dpi=300)
fig.savefig(NomFichier+'.pdf')
